{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% block modal-body-right %}
    <h3>{% trans "Description" %}:</h3>
    <p>
      {% trans "Here can be modified extra-specs for share type." %}<br />
      {% trans "One line - one action. Empty strings will be ignored." %}<br />
      {% trans "To add extra-specs use:" %}
      <pre>key=value</pre>
      {% trans "To unset extra-specs use:" %}
      <pre>key</pre>
      {% trans "All pairs that are in field for left are set for this share type." %}
      {% trans "Extra spec 'driver_handles_share_servers'  is required and should have boolean value." %}<br />
    </p>
{% endblock %}
