{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Share Group Types" %}{% endblock %}

{% block main %}
<div class="row">
  <div class="col-sm-12">
  {{ share_group_types_table.render }}
  </div>
</div>
{% endblock %}
